package de.cnam.jemina.ui;

import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Component;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import de.cnam.jemina.entrypoint.IJeminaApplication;
import de.cnam.jemina.i18n.LocalizationManager;
import de.cnam.jemina.resource.ImgThemeResource;
import de.cnam.jemina.ui.controlls.IControll;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Gestalltet das grundlegende Layout der Seite
 * 
 * @author CNAM Team (Nadine, Mike, Jens)
 * @since 19.01.2013
 */
public class JeminaWindow extends Window{
        
    // Fuer die Uebersetzung:
    private final LocalizationManager locale = LocalizationManager.getLocalizationManager(JeminaWindow.class);
    
    // Logging:
    private final Logger log = LoggerFactory.getLogger(JeminaWindow.class);
    
    // UI:
    VerticalLayout layout;
    private Component content;
    private Component header;
    private Component footer;
    
    // Applikation:
    private final IJeminaApplication app;
    
    // Module:
    private List<IControll> moduls;
    private IControll loginControll;
    private IControll logoutControll;
    

    /**
     * 
     * @param app Die Anwendung in der das Fenster angezeigt werden soll
     * @param loginControll Controller f&uuml;r den Login-Prozess
     * @param logoutControll Controller f&uuml;r den Logout-Prozess
     */
    public JeminaWindow(IJeminaApplication app, IControll loginControll, IControll logoutControll) {
        super();
        
        this.app = app;
        this.loginControll = loginControll;
        this.logoutControll = logoutControll;
        moduls = new ArrayList<>();
        
        setSizeFull();
        setCaption(locale.localize("title"));
    }
    
    /**
     * Erstellt die komplette Seite
     */
    private void buildView() {
        layout = new VerticalLayout();
        layout.setSizeFull();
        layout.setMargin(false);

        if (header == null) {
            header = buildHeader();
        }

        if (footer == null) {
            footer = buildFooter();
        }
                        
        layout.addComponent(header);
        layout.addComponent(content);
        layout.addComponent(footer);
        
        layout.setExpandRatio(content, 100F);
        
        setContent(layout);
    }
    
    /**
     * Setzt den Inhalt der Seite
     * @param content 
     */
    public void setPageContent(Component content) {
        this.content = content;
        buildView();
    }
    
    /**
     * Erstellt die Kopfzeile
     * @return 
     */
    private Component buildHeader() {
        HorizontalLayout headerLayout = new HorizontalLayout();
        headerLayout.setStyleName("header");
        headerLayout.setWidth(100F, UNITS_PERCENTAGE);
        
        HorizontalLayout buttonLayout = new HorizontalLayout();
        headerLayout.addComponent(buttonLayout);
        headerLayout.setExpandRatio(buttonLayout, 100F);
        
        // Module hinzufuegen:
        Button button;
        for (final IControll ctrl : moduls) {
            button = new Button(ctrl.getCaption());
            button.setIcon(new ImgThemeResource(ctrl.getIconName()));
            button.addListener(new ButtonClickListener(ctrl));
            
            buttonLayout.addComponent(button);
        }
        
        
        // Login/Logout Leiste:
        HorizontalLayout loginLayout = new HorizontalLayout();
        loginLayout.setSpacing(true);
        headerLayout.addComponent(loginLayout);
        
        if (app.getUser() == null) {
            // Zeige Login
            button = new Button(loginControll.getCaption());
            button.setIcon(new ImgThemeResource(loginControll.getIconName()));
            button.addListener(new ButtonClickListener(loginControll));
        } else {
            // Zeige Logout
            Label userInfo = new Label(locale.localize("userinfo", null, new String[] { app.getUser().getName() }));
            loginLayout.addComponent(userInfo);
            loginLayout.setComponentAlignment(userInfo, Alignment.MIDDLE_RIGHT);
            
            button = new Button(logoutControll.getCaption());
            button.setIcon(new ImgThemeResource(logoutControll.getIconName()));
            button.addListener(new ButtonClickListener(logoutControll));
        }
        
        //
        loginLayout.addComponent(button);
        loginLayout.setComponentAlignment(button, Alignment.MIDDLE_RIGHT);
        
        return headerLayout;
    }
    
    /**
     * Erstellt die Fussleiste
     * @return 
     */
    private Component buildFooter() {
        HorizontalLayout footerLayout = new HorizontalLayout();
        footerLayout.setStyleName("footer");
        footerLayout.setWidth(100F, UNITS_PERCENTAGE);
        
        // Text
        Label copyright = new Label(locale.localize("copyright"));
        copyright.setSizeFull();
        footerLayout.addComponent(copyright);
        footerLayout.setComponentAlignment(copyright, Alignment.BOTTOM_CENTER);        
        
        return footerLayout;
    }
    
    /**
     * F&uuml;gt der Modul der Navigation hinzu
     * @param ctrl 
     */
    public void addModul(IControll ctrl) {
        moduls.add(ctrl);
        header = null;  // Damit wird dieser neu gebaut
    }
        
    /**
     * Entfernt alle Module aus der Navigation
     */
    public void removeAllModuls() {
        moduls.clear();
        header = null;  // Damit wird dieser neu gebaut
    }
    
    /**
     * Wird aufgerufen, wenn der Anwender auf eine Schaltfl&auml;che klickt
     */
    private class ButtonClickListener implements Button.ClickListener {
        
        private IControll controll;
        
        public ButtonClickListener(IControll controll) {
            super();
            this.controll = controll;
        }
        
        @Override
        public void buttonClick(ClickEvent event) {
            if (log.isDebugEnabled()) {
                log.debug("Klick auf Modul '{}'", controll.getName());
            }

            // Anwendung uber den Klick informieren
            app.openModul(controll.getName(), null, null);
        }
        
    }
}
